// -*- mode: C++; c-file-style: "Stroustrup" -*-
//
// This file is part of krot,
// a program for the simulation, assignment and fit of HRLIF spectra.
//
// Copyright (C) 1998 Jochen Kpper
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; see the file License. if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 , USA
//
// If you use this program for your scientific work, please cite it according to
// the file CITATION that comes with the package.
//
// $Id: krotData.cc,v 1.9 1998/10/09 14:49:20 jochen Exp $



#include "importRawData.h"
#include "krot.h"
#include "krotData.h"
#include "rawData.h"

#include "arnirot/arnirot.h"

#include <climits>
#include <fstream.h>

#include <kdebug.h>



void KRotData::add( const AssignedFrequency& asn )
{
    KROT_LAUNCH( "Launching KRotData::assignment" );
    // try to insert the assignment,
    pair< mapAssignment::iterator, bool > p = ass.insert( pairAssignment( QNum( asn ), asn.frequency() ) );
    if( false == p.second ) {
	KROT_DEBUG_VERBOSE( "  calling ourself recursively." );
	// if it fails, delete the old entry and call ourself recursevly.
	ass.erase( p.first );
	add( asn );
    }
    return;
}



void KRotData::arnirot( const CalculationParameter par )
{
    KROT_LAUNCH( "Launching KRotData::arnirot( const CalculationParameter )" );
    pars = par;
    pars.writeConfig( config );
    Arnirot *calc = new Arnirot( pars, ass );
    connect( calc, SIGNAL( done( const CalculationParameter, vector< Transition > ) ),
	     this, SLOT( arnirot( const CalculationParameter, vector< Transition > ) ) );
    calc->start();
    return;
}



void KRotData::arnirot( const CalculationParameter par, vector< Transition > lines )
{
    KROT_LAUNCH( "Launching KRotData::arnirot( const CalculationParameter, vector< Transition > )" );
    pars = par;
    // create simulation here
    KROT_DEBUG1_VERBOSE( "Number of calculated lines %d", lines.size() );
    Simulation sim( lines, this );
    add( sim );
    return;
}



void KRotData::checkLimits()
{
#ifdef LAUNCH
    cerr << "Launching KRotData::checkLimits" << endl;
#endif
    // check for the frequency range we do have data in
    strt = INT_MAX;
    stp  = INT_MIN;
    // scan simulations
    if( ! sims.empty() ) {
	mapSimulation::iterator iter = sims.begin();
	while( sims.end() != iter ) {
	    if( iter->second.start() < strt )
		strt = iter->second.start();
	    if( iter->second.stop() > stp )
		stp = iter->second.stop();
	    iter++;
	}
    }
    // scan spectra
    if( ! spec.empty() ) {
	mapSpectrum::iterator iter = spec.begin();
	while( spec.end() != iter ) {
	    if(iter->second.start() < strt)
		strt = iter->second.start();
	    if( iter->second.stop() > stp )
		stp = iter->second.stop();
	    iter++;
	}
    }
    // check monotonicy
    if( strt > stp )
	stp = strt;
#ifdef DEBUG
    mapSimulation::const_iterator iter = sims.begin();
    while( sims.end() != iter ) {
	cerr << "krot: " << iter->first << " contains " << iter->second.lines().size() << " lines." << endl;
	iter++;
    }
#endif
    // tell the outside world
    emit limits( strt, stp );
    return;
}



void KRotData::eraseAll()
{
    if( !( spec.empty() || sims.empty() ) ) {
	QString msg = ( i18n( "Do you really want to kill all sets ?" ) );
	int res = KMsgBox::yesNo(0, i18n( "Delete all sets" ), msg,
				 KMsgBox::QUESTION | KMsgBox::DB_SECOND,
				 i18n("Yes"), i18n("No") );
	if( 1 == res) {
	    sims.erase( sims.begin(), sims.end() );
	    spec.erase( spec.begin(), spec.end() );
	    checkLimits();
	    emit message( i18n( "All sets deleted !" ) );
	}
    }
    return;
}



void KRotData::exportAssignments( const string filename ) const
{
    ofstream stream( filename.c_str(), ios::trunc );
    stream.setf( ios::fixed );
    stream.precision( 1 );
    mapAssignment::const_iterator iter = ass.begin();
    while( ass.end() != iter ) {
	stream << string( iter->first ) << "  ";
	stream << iter->second << endl;
	iter++;
    }
    return;
}



void KRotData::exportXY( const QString setName, const QString fileName )
{
    if( sims.end() != sims.find( setName ) )
	sims[ setName ].save( fileName, DataSet::XY_DATA );
    else
	if( spec.end() != spec.find( setName ) )
	    spec[ setName ].save( fileName, DataSet::XY_DATA );
    string msg(" ");
    msg += setName + i18n( " saved." );
    emit message( msg.c_str() );
    return;
}



void KRotData::exportLines( const QString simName, const QString fileName )
{
    return;
}



void KRotData::importLines( const QString filename )
{
    KDEBUG1( KDEBUG_INFO, KDEBUG_KROT_LAUNCH,
	    "Launching KRotData::importLines with filename %s", filename.data() );
    Simulation sim( this );
    sim.load( filename, DataSet::LINES_DAT_DATA );
    add( sim );
    // tell the user
    string msg( i18n( " Simulation " ) );
    msg += sim.shortName() + i18n( " imported" );
    emit message( msg.c_str() );
    return;
}



void KRotData::importJba( const QString filename )
{
    KROT_LAUNCH( "Launching KRotData::importJba with filename " );
    // open file
    QFile file( filename );
    if(! file.open( IO_ReadOnly) ) {
	KMsgBox::message( 0, i18n( "File Error" ),
			  i18n( "Could not open file to import KHiDaq data." ),
			  KMsgBox::EXCLAMATION, i18n( "OK" ) );
	return;
    }
    file.close();
    JbaData *jba = new JbaData( filename );
    QString msg( i18n( " jba data from " ) );
    msg += file.name();
    msg += i18n( " imported" );
    emit message( msg.data() );
    // show raw data + marker positions
#warning Needs to be substituted by jba specific dialog ! (Jochen)
    DialogImportKHiDaq *dialog = new DialogImportKHiDaq( *jba );
    dialog->show();
    connect( dialog, SIGNAL( marker( const int ) ),
	     jba,    SLOT( toggleMarker( const int ) ) );
    // create spectrum from the data

    return;
}



void KRotData::importKHiDaq( const RawData& khidaq )
{
    KDEBUG( KDEBUG_INFO, KDEBUG_KROT_LAUNCH, "Launching KRotData::importKHiDaq( RawData& )" );
    Spectrum spec( khidaq, this );
    add( spec );
    // tell the user
    string msg( i18n( " Spectrum " ) );
    msg += spec.shortName() + i18n( " imported" );
    emit message( msg.c_str() );
    emit newSet();
    return;
}



void KRotData::importXY( const QString filename )
{
    KROT_LAUNCH1( "Launching KRotData::importXY with filename %s", filename.data() );
    Spectrum spectrum( this );
    spectrum.load( filename, DataSet::XY_DATA );
    add( spectrum );
    // tell the user
    QString msg( i18n( " Spectrum " ) );
    msg += spectrum.shortName() + i18n( " imported" );
    emit message( msg.data() );
    emit newSet();
    return;
}
